#!/usr/bin/env python
"""
 run tomte learner 

 usage:  tomte_learn  config.yaml

   where config.yaml  specifies :
      - learning parameters
      - sutinfo file which describes alphabet used by sut 
      - how to connect to sut (default is using socket localhost:7892

"""
# Author: Harco Kuppens


import sys, re, pprint,os  # modules from standard lib (python 2.6 and later)

import tempfile
import shutil
import argparse
import yaml

import util
import learnresult2uppaal


def check_dependencies():
    util.check_program('java','.exe')

def args_parsing(args):

    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    #argparser.add_argument("--max-traces", action='store'  ,help='Maximum number of traces to run during equivalence testing.')
    #argparser.add_argument("--min-trace-length", action='store', help="Minimum length of traces during equivalence query.")
    #argparser.add_argument("--max-trace-length", action='store',help="Maximum length of traces during equivalence query.")

    argparser.add_argument("--seed", action='store', help="Seed to use for random number generator.")	
    argparser.add_argument("--output-dir", action='store',help="Directory to store output.")
    argparser.add_argument('--port',action='store',help='tcp port  to listen on for incoming connections')

    argparser.add_argument('configfile', help='learner configuration file')

    parse_result=argparser.parse_args(args[1:])


    seed=parse_result.seed
    output_dir=parse_result.output_dir
    port=parse_result.port

    # handle input/output file argument
    configfile=os.path.abspath(parse_result.configfile).replace('\\','/')
    util.check_isreadablefile(configfile)

    return  (configfile,output_dir,seed,port)


def learn(configfile,output_dir,seed,port,tomte_root_dir,tomte_classpath):
    """
	 run tomte learner 

	 usage:  tomte_learn  config.yaml

	   where config.yaml  specifies :
	      - learning parameters
	      - sutinfo file which describes alphabet used by sut 
              - how to connect to sut (default is using socket localhost:7892

    """

    # read config and sutinfo files to retreive sut's name
    configdir=os.path.dirname(configfile)
   

    configdata=yaml.load(file(configfile))

    # check first if sutinfoFile is really set in config.yaml file 
    if not configdata.has_key('learning'): 
        print "you must specify learning configuration in configuration"
        sys.exit(1)
    if not configdata['learning'].has_key('sutinfoFile'): 
        print "you must specify sutinfoFile in configuration"
        sys.exit(1)

    # we take path to sutinfoFile to be relative from the directory where the config.yaml is located (configDir)
    # TODO: also allow absolute dir for sutinfoFile
    sutinfoFile=os.path.normpath( os.path.join(configdir, configdata['learning']['sutinfoFile']) )
    if not util.isreadablefile(sutinfoFile):
        print "\nPROBLEM:\n   cannot read the file : " + sutinfoFile +"\n\n"
        sys.exit(1)
         
    #sutinfoFile=configdata['learning']['sutinfoFile']

    sutinfoData=yaml.load(file(sutinfoFile))
    

    sutname=sutinfoData['name']
    import time
    timestamp=time.strftime("%y-%m-%d_%H.%M.%S")


    # set parameterized output dir here so that after learnig we know were to access the results (MUST now timetamp!!) 
    if output_dir:
       outputDir=output_dir
    else:
       if configdata.has_key('learnResults') and configdata['learnResults'].has_key('outputDir'):
          outputDir=configdata['learnResults']['outputDir']
       else:
          outputDir="output/${timestamp}_${sutname}"
    outputDir=outputDir.replace("${timestamp}",timestamp).replace("${sutname}",sutname)


    # 1. run tomte.jar 
    #----------------------------
    classpath=[
        tomte_classpath,
        "lib/learnlib-distribution-20110714.jar",
        "lib/api-0.4.7.jar",
        "lib/json_simple-1.1.jar",
        "lib/snakeyaml-1.9.jar",
        "lib/log4j-1.2.17.jar",
        "lib/jbeandumper.1.5.1.jar",  
    ] 
    classpath = [ os.path.abspath(os.path.join(tomte_root_dir,entry)) for entry in classpath ]
    classpath = os.path.pathsep.join(classpath)
    #command=[ 'java', '-cp', classpath, 'abslearning.learner.Main','--seed',str(seed),'--output-dir',outputDir,'--port',str(port),configfile]
    command=[ 'java', '-cp', classpath, 'abslearning.learner.Main','--tomte-root-path',tomte_root_dir,'--output-dir',str(outputDir)]
    # pass through optionally commands 
    if seed: command+=['--seed',str(seed)]
    if port: command+=['--port',str(port)]
    command.append(configfile)
    #util.run_command(command)
    

    print("running command : \n   "  + util.join_command(command) )
    import subprocess
    try:
      process = subprocess.Popen(command)
    except Exception as inst:
      print("problem running command : \n   "  + util.join_command(command) + "\n problem : " + str(inst))
    
    print 'learning'
    process.communicate()  # no pipes set for stdin/stdout/stdout streams so does effectively only just wait for process ends  (same as process.wait()

    if process.returncode:
       print "problem:  learner exited with errorcode " + str(process.returncode)
       sys.exit(process.returncode)  
    print 'learning done'

    # 2. make uppaal model from learnresults 
    #---------------------------------------

    # read dumped config file by learner -> reason : in this file all defaults are explicit set
    configdata=yaml.load(file(outputDir + '/input/config.yaml'))
    json_filename_in = outputDir + '/' + configdata['learnResults']['learnedModelDataFile']
    dot_filename_in  = outputDir + '/' + configdata['learnResults']['abstractModelDotFile']
    model_file_out   = outputDir + '/' + configdata['learnResults']['learnedConcreteModelFile']

    learnresult2uppaal.learnresult2uppaal(json_filename_in,dot_filename_in,model_file_out)


    print "\n\n"
    print "results are stored in directory : " + outputDir 
    print "learned concrete model: " +  model_file_out 
    print "\n\n"





def main(args):
    """
	 run tomte learner 

	 usage:  tomte_learn  config.yaml

	   where config.yaml  specifies :
	      - learning parameters
	      - sutinfo file which describes alphabet used by sut 
              - how to connect to sut (default is using socket localhost:7892


    """
    check_dependencies()

    (configfile,output_dir,seed,port)=args_parsing(args)
    current_script_dir=os.path.dirname(sys.argv[0])
    tomte_root_dir= os.path.normpath(os.path.join(current_script_dir,'..') )
    tomte_classpath="lib/tomte-0.2.jar";
    learn(configfile,output_dir,seed,port,tomte_root_dir,tomte_classpath)



if __name__ == "__main__":
    print main(sys.argv)

