#!/usr/bin/env python
"""
 run tomte learner 

 usage:  tomte_learn_in_eclipse  config.yaml

   where config.yaml  specifies :
      - learning parameters
      - sutinfo file which describes alphabet used by sut 
      - how to connect to sut (default is using socket localhost:7892

"""
# Author: Harco Kuppens


import sys, re, pprint,os  # modules from standard lib (python 2.6 and later)

import tempfile
import shutil
import argparse
import yaml

import util
import learn

def check_dependencies():
    util.check_program('java','.exe')

def args_parsing(args):

    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    #argparser.add_argument("--max-traces", action='store'  ,help='Maximum number of traces to run during equivalence testing.')
    #argparser.add_argument("--min-trace-length", action='store', help="Minimum length of traces during equivalence query.")
    #argparser.add_argument("--max-trace-length", action='store',help="Maximum length of traces during equivalence query.")

    argparser.add_argument("--seed", action='store', help="Seed to use for random number generator.")	
    argparser.add_argument("--output-dir", action='store',help="Directory to store output.")
    argparser.add_argument('--port',action='store',help='tcp port  to listen on for incoming connections')

    argparser.add_argument('configfile', help='learner configuration file')

    parse_result=argparser.parse_args(args[1:])


    seed=parse_result.seed
    output_dir=parse_result.output_dir
    port=parse_result.port

    # handle input/output file argument
    configfile=os.path.abspath(parse_result.configfile).replace('\\','/')
    util.check_isreadablefile(configfile)

    return  (configfile,output_dir,seed,port)




def main(args):
    """
	 run tomte learner 

	 usage:  tomte_learn_in_eclipse  config.yaml

	   where config.yaml  specifies :
	      - learning parameters
	      - sutinfo file which describes alphabet used by sut 
              - how to connect to sut (default is using socket localhost:7892


    """
    check_dependencies()

    (configfile,output_dir,seed,port)=args_parsing(args)
    current_script_dir=os.path.dirname(sys.argv[0])
    tomte_root_dir= os.path.normpath(os.path.join(current_script_dir,'..') )
    tomte_classpath="build";
    learn.learn(configfile,output_dir,seed,port,tomte_root_dir,tomte_classpath)



if __name__ == "__main__":
    print main(sys.argv)

