package modeltest;



import org.junit.AfterClass;
import org.junit.Before;
import org.junit.Test;

import abslearning.learner.Config;
import java.io.FileNotFoundException;


// junit
//  - to run each test method each time a new instance of the testObject is made
//     -> only static class fields are shareable between tests!
//  - method with @Before is run before each test (in a  new instance)
//  - static method with @BeforeClass is run once on class before all tests start and is
//    persistent!
// in eclipse 
//  - time spend in Before and After methods is included in test timings
//  - time spend in BeforeClass and AfterClass  static methods is not included in any test timing
//    because they are used for all tests combined!

// tricks used :
//   - run what you normally run in @beforeclass in a separate test instead 
//     and make that test run by a @before method and that test only runs one by checking a 
//     static variable is set or not 
//     Advantages :
//       * what initially was run in a beforeClass run is now a test 
//       * which you can time!!
//   - implement tests as static methods in a Util class
//       those utility functions can then be used more easily in other type of tests
//       Note: it is more difficult in UnitTest object testMethod to run a testMethod of another unitTest  



public class BasicTest {
	protected static String referenceDir="unknown";
    protected static  Config config;
    protected static boolean learningDone = false;
    protected static String inheritedClassName="unknown";
    protected static String modelsDir="models/";
    
    @Before
	public  void before() throws FileNotFoundException {	
    	// check for switch inherited class!!
    	String className;
    	className=this.getClass().getSimpleName();
    	if ( ! inheritedClassName.equals(className) ) {
    		inheritedClassName=className;
        	//referenceDir="test/regression/modeltest/resources/" + inheritedClassName + "/";
    		String modelName=inheritedClassName.replaceFirst("Test_", "");
    		referenceDir=modelsDir + modelName + "/";
        	learningDone = false;
    	}
    	// do learning first
    	this.testLearn();  // note: effectively done only at first call!
	}	 
    
    @AfterClass
    public static void afterClass () {
    	learningDone = false;
    	inheritedClassName="unknown";
    	referenceDir="unknown";
    	config=null;
    }
	    
    
    
	@Test
	public void testLearn() throws FileNotFoundException {			     		
		if (  ! learningDone ) {    	
     		 config=Util.learn(referenceDir);  //,1299777356022L  
     		 learningDone = true;
        }    		
	}
    
	
	@Test
	public void testConcreteModel() throws FileNotFoundException {		
		Util.compareConcreteModels(referenceDir,config);  
	}		
	
	@Test
	public void testAbstraction() throws FileNotFoundException {	
		Util.compareAbstractions(referenceDir,config); 
	}
	
	@Test
	public void testAbstractModel() throws FileNotFoundException {	
		// start learner
		Util.compareAbstractModels(referenceDir,config); 
	}	

	
	// TODO:
/*	
	@Test
	public void testMembershipQuery() throws FileNotFoundException {	
		// start learner
		Util.compareMembershipQueries(referenceDir,config);  
	}
*/
	

	
	
	
	/*
	@Test
	public void testBatch() throws FileNotFoundException {
		
	    long[] seeds={
	       1299777356022L,				
	       1299777356021L,
	       1299777356020L,
	       //1299777356022L
	    };
		
		for ( long seed : seeds ) {		
			// start learner
			Util.runTestSpecifiedInDir(testDir,seed);
		}		
	}	
	*/

 	
	
}