package modeltest;

import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Date;

import util.NullOutputStream;
import util.RunCmd;


import abslearning.learner.Config;

public class Util {
	public static Config learn(String testDir) throws FileNotFoundException {
		return learn(testDir, 0);
	}



	public static Config learn(String referenceDir,long seed) throws FileNotFoundException {

		// for unit testing use tmp dir 
		String tmpDir = "tmp/regressiontest/"; 
		

		System.err.println("start new unittest "  );
		System.err.println("------------------");

		// read in params needed for learning
		
		String configFilepath=referenceDir + "config.yaml" ;
		String[] args={configFilepath};
		Config config=new Config(args); 
		config.learnResults_outputDir=tmpDir+"${timestamp}_${sutname}";  
		if ( seed != 0 ) {
			config.learning_seed=seed;
		}
		
		// check for modelfile which we need for simulation test
		if ( config.sutImplementation_modelFile == null ||  config.sutImplementation_modelFile.equals("") )  {
			// try first to find model.xml next to config file!!
			String configDir=(new File(config.original_configFile)).getParent();
			if ( (new File(configDir + "/model.xml")).canRead() ) {
				config.sutImplementation_modelFile=configDir + "/model.xml";
		    	System.err.println("Warning: for sut simulation a modelFile should be set in the config file");
				System.err.println("         Found model.xml file next to config file, thus take that as modelfile"); 
			} else {
		    	System.err.println("Error: for sut simulation a modelFile must be set in the config file");
			    System.exit(1);
			}
		}
		
		
		
		
		// in production  : socket server testing
		// in development : directwrapper testing 
		
		if (  config.devel_modeOn ) {
			 config.sutInterface_sutWrapperClassName="GeneratedSutDirectWrapper";
		} else {   
			long secsSince1970=  (new Date()).getTime()/1000;
			int uniqNumberUnder1000= (new Long(secsSince1970 % 1000)).intValue();
			int port=8001 + uniqNumberUnder1000; 
			config.sutInterface_portNumber =port; 
			config.sutInterface_sutWrapperClassName="SutSocketWrapper";
			String extension="";
			if ( util.Check.isOsWindows() ) extension=".bat"; 			
			//String[] cmdarray={ "/home/dummy6/workspace/AbsLearnRGB_tree/sut/bin/sut_run" + extension, "--verbose","--port", Integer.toString(port),config.sutImplementation_modelFile};
			
// TODO: check sut_run in path or give error!			
			
			String[] cmdarray={ "/home/dummy6/workspace/sut/bin/sut_run" + extension,"--port", Integer.toString(port),config.sutImplementation_modelFile};
			
			//String[] cmdarray={ "/home/dummy6/workspace/sut/bin/sut_run" + extension,"--port", Integer.toString(port),config.sutImplementation_modelFile};
			
			//RunCmd.runCmd(cmdarray,new NullOutputStream(),false);
			RunCmd.runCmd(cmdarray,System.err,false);
			//RunCmd.runCmd(cmdarray,null,true);
			util.Time.millisleep(2000); // sleep 2 seconds to be sure sut socket server is running!
		
		}
		
		
		//System.exit(1);
		
		// do learning
	   abslearning.learner.Main.learn(config);			
	   
	   
		
		return config;
	}	
	
	public static void compareConcreteModels(String referenceDir, Config config) {		
		boolean  result;
		result=abslearning.learner.Main.makeAndCompareModel(config);
		String msg = "\n see console for counter example trace";
		if (!result) fail(msg);
	}     
	
	public static void compareAbstractModels(String referenceDir, Config config) {
		// compare abstract model learned
		String filename,filepath1, filepath2;
		boolean  result;
		
		filename=config.learnResults_abstractModelDotFile;
		filepath1 = referenceDir + filename;
		
		System.err.println(config.learnResults_outputDir);
		
		filepath2 = config.learnResults_outputDir + filename;
		result = util.Filesystem.cmpText(filepath1, filepath2);
		System.err.println(filepath1 + " equals " + filepath2 + " : " + result);

		String msg = "\nseed: " + config.learning_seed;
		if (!result) fail(msg);		
		
	}


	public static void compareAbstractions(String referenceDir, Config config) {
		// compare abstractions learned		
		String filename,filepath1, filepath2;
		boolean  result;		
		filename=config.learnResults_abstractionFile;
		filepath1 = referenceDir + filename;
		filepath2 = config.learnResults_outputDir + filename;
		result = util.Filesystem.cmpText(filepath1, filepath2);
		
		System.err.println(filepath1 + " equals " + filepath2 + " : " + result);
		
		String msg = "\nseed: " + config.learning_seed;
		if (!result) fail(msg);			
      
		
	}


	public static void compareMembershipQueries(String referenceDir,
			Config config) {
		// TODO Auto-generated method stub
		
	}







}
